@@ -44,6 +44,7 @@ import getpass
 import uuid
 from struct import unpack
 from collections import deque, namedtuple
+import random
 
 # noinspection PyCompatibility,PyUnresolvedReferences
 from six import raise_from, string_types, integer_types, PY2
@@ -53,6 +54,11 @@ if PY2:
 else:
     from urllib.parse import urlparse, parse_qs
 
+    from typing import TYPE_CHECKING
+    if TYPE_CHECKING:
+        from typing import Any, Dict, Literal, Optional, Type, Union
+        from typing_extensions import Self
+
 import vertica_python
 from .. import errors
 from ..vertica import messages
@@ -64,11 +70,14 @@ from ..vertica.log import VerticaLogging
 DEFAULT_HOST = 'localhost'
 DEFAULT_PORT = 5433
 DEFAULT_PASSWORD = ''
+DEFAULT_DATABASE = ''
 DEFAULT_AUTOCOMMIT = False
 DEFAULT_BACKUP_SERVER_NODE = []
 DEFAULT_KRB_SERVICE_NAME = 'vertica'
 DEFAULT_LOG_LEVEL = logging.WARNING
 DEFAULT_LOG_PATH = 'vertica_python.log'
+DEFAULT_BINARY_TRANSFER = False
+DEFAULT_REQUEST_COMPLEX_TYPES = True
 try:
     DEFAULT_USER = getpass.getuser()
 except Exception as e:
@@ -77,6 +86,7 @@ except Exception as e:
 
 
 def connect(**kwargs):
+    # type: (Any) -> Connection
     """Opens a new connection to a Vertica database."""
     return Connection(kwargs)
 
@@ -111,7 +121,8 @@ def parse_dsn(dsn):
         elif key == 'backup_server_node':
             continue
         elif key in ('connection_load_balance', 'use_prepared_statements',
-                     'disable_copy_local', 'ssl', 'autocommit'):
+                     'disable_copy_local', 'ssl', 'autocommit',
+                     'binary_transfer', 'request_complex_types'):
             lower = value.lower()
             if lower in ('true', 'on', '1'):
                 result[key] = True
@@ -163,7 +174,6 @@ class _AddressList(object):
                            ' must be a host string or a (host, port) tuple')
                 self._logger.error(err_msg)
                 raise TypeError(err_msg)
-
         self._logger.debug('Address list: {0}'.format(list(self.address_deque)))
 
     def _append(self, host, port):
@@ -210,7 +220,8 @@ class _AddressList(object):
                 return entry.data
             else:
                 # DNS resolve a single host name to multiple IP addresses
-                self.address_deque.popleft()
+                self.pop()
+                # keep host and port info for adding address entry to deque once it has been resolved
                 host, port = entry.host, entry.data
                 try:
                     resolved_hosts = socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM)
@@ -219,7 +230,8 @@ class _AddressList(object):
                     continue
 
                 # add resolved addrinfo (AF_INET and AF_INET6 only) to deque
-                for addrinfo in reversed(resolved_hosts):
+                random.shuffle(resolved_hosts)
+                for addrinfo in resolved_hosts:
                     if addrinfo[0] in (socket.AF_INET, socket.AF_INET6):
                         self.address_deque.appendleft(_AddressEntry(
                             host=host, resolved=True, data=addrinfo))
@@ -243,12 +255,14 @@ def _generate_session_label():
 
 class Connection(object):
     def __init__(self, options=None):
+        # type: (Optional[Dict[str, Any]]) -> None
         self.parameters = {}
         self.session_id = None
         self.backend_pid = None
         self.backend_key = None
         self.transaction_status = None
         self.socket = None
+        self.socket_as_file = None
 
         options = options or {}
         self.options = parse_dsn(options['dsn']) if 'dsn' in options else {}
@@ -277,7 +291,7 @@ class Connection(object):
                 msg = 'Connection option "user" is required'
                 self._logger.error(msg)
                 raise KeyError(msg)
-        self.options.setdefault('database', self.options['user'])
+        self.options.setdefault('database', DEFAULT_DATABASE)
         self.options.setdefault('password', DEFAULT_PASSWORD)
         self.options.setdefault('autocommit', DEFAULT_AUTOCOMMIT)
         self.options.setdefault('session_label', _generate_session_label())
@@ -305,21 +319,31 @@ class Connection(object):
         self._logger.debug('COPY LOCAL operation is {}'.format(
                      'disabled' if self.options['disable_copy_local'] else 'enabled'))
 
+        # knob for using binary transfer format or text transfer format
+        self.options.setdefault('binary_transfer', DEFAULT_BINARY_TRANSFER)
+        self._logger.debug('Data binary transfer is {}'.format(
+                     'enabled' if self.options['binary_transfer'] else 'disabled'))
+
+        # knob for requesting complex types metadata
+        self.options.setdefault('request_complex_types', DEFAULT_REQUEST_COMPLEX_TYPES)
+        self._logger.debug('Complex types metadata is {}'.format(
+                     'requested' if self.options['request_complex_types'] else 'not requested'))
+
         self._logger.info('Connecting as user "{}" to database "{}" on host "{}" with port {}'.format(
                      self.options['user'], self.options['database'],
                      self.options['host'], self.options['port']))
         self.startup_connection()
 
-        # Initially, for a new session, autocommit is off
-        if self.options['autocommit']:
-            self.autocommit = True
-
+        # Complex types metadata is returned since protocol version 3.12
+        self.complex_types_enabled = self.parameters['protocol_version'] >= (3 << 16 | 12) and \
+                                     self.parameters.get('request_complex_types', 'off') == 'on'
         self._logger.info('Connection is ready')
 
     #############################################
     # supporting `with` statements
     #############################################
     def __enter__(self):
+        # type: () -> Self
         return self
 
     def __exit__(self, type_, value, traceback):
@@ -350,6 +374,7 @@ class Connection(object):
         cur.execute('ROLLBACK;')
 
     def cursor(self, cursor_type=None):
+        # type: (Self, Optional[Union[Literal['list', 'dict'], Type[list[Any]], Type[dict[Any, Any]]]]) -> Cursor
         if self.closed():
             raise errors.ConnectionError('Connection is closed')
 
@@ -366,6 +391,7 @@ class Connection(object):
     @property
     def autocommit(self):
         """Read the connection's AUTOCOMMIT setting from cache"""
+        # For a new session, autocommit is off by default
         return self.parameters.get('auto_commit', 'off') == 'on'
 
     @autocommit.setter
@@ -419,6 +445,7 @@ class Connection(object):
         self.backend_key = None
         self.transaction_status = None
         self.socket = None
+        self.socket_as_file = None
         self.address_list = _AddressList(self.options['host'], self.options['port'],
                                          self.options['backup_server_node'], self._logger)
 
@@ -445,6 +472,11 @@ class Connection(object):
         self.socket = raw_socket
         return self.socket
 
+    def _socket_as_file(self):
+        if self.socket_as_file is None:
+            self.socket_as_file = self._socket().makefile('rb')
+        return self.socket_as_file
+
     def create_socket(self, family):
         """Create a TCP socket object"""
         raw_socket = socket.socket(family, socket.SOCK_STREAM)
@@ -587,6 +619,8 @@ class Connection(object):
         try:
             if self.socket is not None:
                 self._socket().close()
+            if self.socket_as_file is not None:
+                self._socket_as_file().close()
         finally:
             self.reset_values()
 
@@ -648,6 +682,8 @@ class Connection(object):
                     else:
                         # The rest of the message is read later with write_to_disk()
                         message = messages.WriteFile(filename, file_length)
+                elif type_ == messages.RowDescription.message_id:
+                    message = BackendMessage.from_type(type_, self.read_bytes(size - 4), complex_types_enabled=self.complex_types_enabled)
                 else:
                     message = BackendMessage.from_type(type_, self.read_bytes(size - 4))
                 self._logger.debug('<= %s', message)
@@ -686,19 +722,19 @@ class Connection(object):
 
     def read_bytes(self, n):
         if n == 1:
-            result = self._socket().recv(1)
+            result = self._socket_as_file().read(1)
             if not result:
                 raise errors.ConnectionError("Connection closed by Vertica")
             return result
         else:
-            buf = bytearray(n)
-            view = memoryview(buf)
+            buf = b""
             to_read = n
             while to_read > 0:
-                received = self._socket().recv_into(view, to_read)
+                data = self._socket_as_file().read(to_read)
+                received = len(data)
                 if received == 0:
                     raise errors.ConnectionError("Connection closed by Vertica")
-                view = view[received:]
+                buf += data
                 to_read -= received
             return buf
 
@@ -778,8 +814,11 @@ class Connection(object):
         session_label = self.options['session_label']
         os_user_name = DEFAULT_USER if DEFAULT_USER else ''
         password = self.options['password']
+        autocommit = self.options['autocommit']
+        binary_transfer = self.options['binary_transfer']
+        request_complex_types = self.options['request_complex_types']
 
-        self.write(messages.Startup(user, database, session_label, os_user_name))
+        self.write(messages.Startup(user, database, session_label, os_user_name, autocommit, binary_transfer, request_complex_types))
 
         while True:
             message = self.read_message()
