@@ -60,10 +60,21 @@ import six
 from six import binary_type, text_type, string_types, integer_types, BytesIO, StringIO
 from six.moves import zip
 
+if six.PY3:
+    from typing import TYPE_CHECKING
+
+    if TYPE_CHECKING:
+        from typing import IO, Any, AnyStr, Callable, Dict, Generator, List, Literal, Optional, Sequence, Tuple, Type, TypeVar, Union
+        from typing_extensions import Self
+        from .connection import Connection
+        from logging import Logger
+        T = TypeVar('T')
+
 from .. import errors, os_utils
 from ..compat import as_text
 from ..vertica import messages
 from ..vertica.column import Column
+from ..vertica.deserializer import Deserializer
 
 
 # A note regarding support for temporary files:
@@ -136,6 +147,7 @@ class Cursor(object):
     _insert_statement = re.compile(RE_BASIC_INSERT_STAT, re.U | re.I)
 
     def __init__(self, connection, logger, cursor_type=None, unicode_error=None):
+        # type: (Connection, Logger, Optional[Union[Literal['list', 'dict'], Type[list[Any]], Type[dict[Any, Any]]]], Optional[str]) -> None
         self.connection = connection
         self._logger = logger
         self.cursor_type = cursor_type
@@ -147,6 +159,8 @@ class Cursor(object):
         self.prepared_name = "s0"
         self.error = None
         self._sql_literal_adapters = {}
+        self._disable_sqldata_converter = False
+        self._des = Deserializer()
 
         #
         # dbapi attributes
@@ -159,6 +173,7 @@ class Cursor(object):
     # supporting `with` statements
     #############################################
     def __enter__(self):
+        # type: () -> Self
         return self
 
     def __exit__(self, type_, value, traceback):
@@ -179,6 +194,7 @@ class Cursor(object):
 
     def execute(self, operation, parameters=None, use_prepared_statements=None,
                 copy_stdin=None, buffer_size=DEFAULT_BUFFER_SIZE):
+        # type: (str, Optional[Union[List[Any], Tuple[Any], Dict[str, Any]]], Optional[bool], Optional[Union[IO[AnyStr], List[IO[AnyStr]]]], int) -> Self
         if self.closed():
             raise errors.InterfaceError('Cursor is closed')
 
@@ -224,6 +240,8 @@ class Cursor(object):
         return self
 
     def executemany(self, operation, seq_of_parameters, use_prepared_statements=None):
+        # type: (str, Sequence[Union[List[Any], Tuple[Any], Dict[str, Any]]], Optional[bool]) -> None
+
         if not isinstance(seq_of_parameters, (list, tuple)):
             raise TypeError("seq_of_parameters should be list/tuple")
 
@@ -276,6 +294,7 @@ class Cursor(object):
                     "executemany is implemented for simple INSERT statements only")
 
     def fetchone(self):
+        # type: () -> Optional[Union[List[Any], OrderedDict[str, Any]]]
         while True:
             if isinstance(self._message, messages.DataRow):
                 if self.rowcount == -1:
@@ -287,7 +306,11 @@ class Cursor(object):
                 self._message = self.connection.read_message()
                 return row
             elif isinstance(self._message, messages.RowDescription):
-                self.description = [Column(fd, self.unicode_error) for fd in self._message.fields]
+                self.description = self._message.get_description()
+                self._deserializers = self._des.get_row_deserializers(self.description,
+                                        {'unicode_error': self.unicode_error,
+                                         'session_tz': self.connection.parameters.get('timezone', 'unknown'),
+                                         'complex_types_enabled': self.connection.complex_types_enabled,})
             elif isinstance(self._message, messages.ReadyForQuery):
                 return None
             elif isinstance(self._message, END_OF_RESULT_RESPONSES):
@@ -309,6 +332,7 @@ class Cursor(object):
             self._message = self.connection.read_message()
 
     def fetchmany(self, size=None):
+        # type: (Optional[int]) -> List[Union[List[Any], OrderedDict[str, Any]]]
         if not size:
             size = self.arraysize
         results = []
@@ -325,6 +349,7 @@ class Cursor(object):
         return list(self.iterate())
 
     def nextset(self):
+        # type: () -> bool
         """
         Skip to the next available result set, discarding any remaining rows
         from the current result set.
@@ -342,7 +367,11 @@ class Cursor(object):
             # there might be another set, read next message to find out
             self._message = self.connection.read_message()
             if isinstance(self._message, messages.RowDescription):
-                self.description = [Column(fd, self.unicode_error) for fd in self._message.fields]
+                self.description = self._message.get_description()
+                self._deserializers = self._des.get_row_deserializers(self.description,
+                                        {'unicode_error': self.unicode_error,
+                                         'session_tz': self.connection.parameters.get('timezone', 'unknown'),
+                                         'complex_types_enabled': self.connection.complex_types_enabled,})
                 self._message = self.connection.read_message()
                 if isinstance(self._message, messages.VerifyFiles):
                     self._handle_copy_local_protocol()
@@ -379,6 +408,7 @@ class Cursor(object):
     # non-dbapi methods
     #############################################
     def closed(self):
+        # type: () -> bool
         return self._closed or self.connection.closed()
 
     def cancel(self):
@@ -389,12 +419,14 @@ class Cursor(object):
             'to cancel the current database operation.')
 
     def iterate(self):
+        # type: () -> Generator[Union[List[Any], OrderedDict[str, Any]], None, None]
         row = self.fetchone()
         while row:
             yield row
             row = self.fetchone()
 
     def copy(self, sql, data, **kwargs):
+        # type: (str, IO[AnyStr], Any) -> None
         """
 
         EXAMPLE:
@@ -459,10 +491,23 @@ class Cursor(object):
         return self.object_to_string(py_obj, False)
 
     def register_sql_literal_adapter(self, obj_type, adapter_func):
+        # type: (T, Callable[[T], str]) -> None
         if not callable(adapter_func):
             raise TypeError("Cannot register this sql literal adapter. The adapter is not callable.")
         self._sql_literal_adapters[obj_type] = adapter_func
 
+    @property
+    def disable_sqldata_converter(self):
+        return self._disable_sqldata_converter
+
+    @disable_sqldata_converter.setter
+    def disable_sqldata_converter(self, value):
+        """By default, the client does data conversions for query results:
+        reading a bytes sequence from server and creating a Python object out of it.
+        If set to True, bypass conversions from SQL type raw data to the native Python object
+        """
+        self._disable_sqldata_converter = bool(value)
+
     #############################################
     # internal
     #############################################
@@ -506,14 +551,19 @@ class Cursor(object):
             raise TypeError('Unrecognized cursor_type: {0}'.format(self.cursor_type))
 
     def format_row_as_dict(self, row_data):
+        if self._disable_sqldata_converter:
+            return OrderedDict((descr.name, value)
+                    for descr, value in zip(self.description, row_data.values))
         return OrderedDict(
-            (descr.name, descr.convert(value))
-            for descr, value in zip(self.description, row_data.values)
+            (descr.name, convert(value))
+            for descr, convert, value in zip(self.description, self._deserializers, row_data.values)
         )
 
     def format_row_as_array(self, row_data):
-        return [descr.convert(value)
-                for descr, value in zip(self.description, row_data.values)]
+        if self._disable_sqldata_converter:
+            return row_data.values
+        return [convert(value)
+                for convert, value in zip(self._deserializers, row_data.values)]
 
     def object_to_string(self, py_obj, is_copy_data):
         """Return the SQL representation of the object as a string"""
@@ -581,7 +631,11 @@ class Cursor(object):
 
     def format_quote(self, param, is_copy_data):
         if is_copy_data:
-            return u'"{0}"'.format(re.escape(param))
+            s = list(param)
+            for i, c in enumerate(param):
+                if c in u'()[]{}?"*+-|^$\\.&~# \t\n\r\v\f':
+                    s[i] = "\\" + c
+            return u'"{0}"'.format(u"".join(s))
         else:
             return u"'{0}'".format(param.replace(u"'", u"''"))
 
@@ -605,7 +659,11 @@ class Cursor(object):
         if isinstance(self._message, messages.ErrorResponse):
             raise errors.QueryError.from_error_response(self._message, query)
         elif isinstance(self._message, messages.RowDescription):
-            self.description = [Column(fd, self.unicode_error) for fd in self._message.fields]
+            self.description = self._message.get_description()
+            self._deserializers = self._des.get_row_deserializers(self.description,
+                                    {'unicode_error': self.unicode_error,
+                                     'session_tz': self.connection.parameters.get('timezone', 'unknown'),
+                                     'complex_types_enabled': self.connection.complex_types_enabled,})
             self._message = self.connection.read_message()
             if isinstance(self._message, messages.ErrorResponse):
                 raise errors.QueryError.from_error_response(self._message, query)
@@ -795,7 +853,11 @@ class Cursor(object):
         if isinstance(self._message, messages.NoData):
             self.description = None  # response was NoData for a DDL/transaction PreparedStatement
         else:
-            self.description = [Column(fd, self.unicode_error) for fd in self._message.fields]
+            self.description = self._message.get_description()
+            self._deserializers = self._des.get_row_deserializers(self.description,
+                                    {'unicode_error': self.unicode_error,
+                                     'session_tz': self.connection.parameters.get('timezone', 'unknown'),
+                                     'complex_types_enabled': self.connection.complex_types_enabled,})
 
         # Read expected message: CommandDescription
         self._message = self.connection.read_expected_message(messages.CommandDescription, self._error_handler)
@@ -832,7 +894,8 @@ class Cursor(object):
                            .format(parameter_values, len(parameter_values), parameter_count))
                     raise ValueError(msg)
                 self.connection.write(messages.Bind(portal_name, self.prepared_name,
-                                             parameter_values, parameter_type_oids))
+                                             parameter_values, parameter_type_oids,
+                                             self.connection.options['binary_transfer']))
                 self.connection.write(messages.Execute(portal_name, 0))
             self.connection.write(messages.Sync())
         except Exception as e:
